//
//  main.swift
//  Terminal
//
//  Created by Paul Hudson on 30/05/2021.
//

import Foundation

func anyToString(_ thing: Any) -> String {
    String(describing: thing)
}

extension Sequence {
    func transformed<NewType>(_ converter: (Element) throws -> NewType) rethrows -> [NewType] {
        var result = [NewType]()
        result.reserveCapacity(self.underestimatedCount)

        for item in self {
            let transformed = try converter(item)
            result.append(transformed)
        }

        return result
    }
}

let numbers1 = [1, 2, 3]
let result1 = numbers1.map(Double.init)
print(result1)

let numbers2 = [1: "One", 2: "Two", 3: "Three"]
let result2 = numbers2.map { element in
    (element.key, element.value)
}
print(result2)

let squares: [Double] = [4, 9, 16, 25, 36, 49, 64, 81, 100]
let roots = squares.map(sqrt)
print(roots)

func fibonacci(of number: Int) -> Int {
    var first = 0
    var second = 1

    for _ in 0..<number {
        let previous = first
        first = second
        second = previous + first
    }

    return first
}

let first50 = (0..<50).map(fibonacci)
print(first50)

let names = ["Taylor", "Justin", "Adele"]

let nameCounts1 = names.map { name in
    name.count
}

print(nameCounts1)

let nameCounts2 = names.map(\.count)
print(nameCounts2)

//enum Optional<Wrapped> {
//    case none
//    case some(Wrapped)
//}
//
//enum Result<Success, Failure: Error> {
//    case success(Success)
//    case failure(Failure)
//}

//func getSavedData() -> Data? {
//    Data("Saved data goes here".utf8)
//}
//
//let savedString = getSavedData().map { data in
//    String(decoding: data, as: UTF8.self)
//}

func getSavedData() -> Result<Data, Error> {
    .success(Data("Saved data goes here".utf8))
}

let savedString = getSavedData().map { data in
    String(decoding: data, as: UTF8.self)
}

let scores = [100, 80, 85]
let mappedScores = scores.map { $0 }
print(scores == mappedScores)

func uppercased(_ string: String) -> String {
    string.uppercased()
}

func reversed(_ string: String) -> String {
    String(string.reversed())
}

func uppercasedAndReversed(_ string: String) -> String {
    String(string.uppercased().reversed())
}

let names1 = names.map(uppercased).map(reversed)
let names2 = names.map(uppercasedAndReversed)
print(names1 == names2)
